import argparse
import os

import pyperclip

from . import __version__ as version


def get_code_from_file(filename: str) -> str:
    if not os.path.isfile(filename):
        raise argparse.ArgumentTypeError(f"No such file: {filename}")
    with open(os.path.abspath(filename), mode="rt") as fp:
        return fp.read()


def environ_or_required(key: str) -> dict:
    env_var = os.environ.get(key)
    if env_var is None:
        return {"required": True}
    else:
        return {"default": env_var}


def get_args():
    parser = argparse.ArgumentParser(
        description="Create a carbon code image",
        formatter_class=argparse.ArgumentDefaultsHelpFormatter,
    )
    parser.add_argument(
        "-v", "--version", action="version", version=f"%(prog)s {version}"
    )
    group = parser.add_mutually_exclusive_group(required=True)
    group.add_argument(
        "-f", "--file", type=get_code_from_file, help="File with code", dest="code"
    )
    group.add_argument(
        "-c",
        "--clipboard",
        help="Use code on clipboard",
        action="store_const",
        const=pyperclip.paste(),
        dest="code",
    )
    group.add_argument("-s", "--snippet", help="Code snippet", dest="code")

    parser.add_argument(
        "-i",
        "--interactive",
        action="store_true",
        default=False,
        help="Run Selenium in interactive (not headless) mode",
    )
    parser.add_argument(
        "-l", "--language", help="Programming language", default="python"
    )
    parser.add_argument(
        "-b", "--background", help="Background color", default="#ABB8C3"
    )
    parser.add_argument("-t", "--theme", help="Name of the theme", default="seti")
    parser.add_argument(
        "-d",
        "--destination",
        default=os.getcwd(),
        help="Specify folder where image should be stored (defaults to current directory)",
    )
    parser.add_argument("-w", "--wt", help="Windows control theme", default="sharp")
    parser.add_argument(
        "--driver-path",
        help="Path to the executable, if it is not given it reads value from environment variable (DRIVER_PATH)",
        **environ_or_required("DRIVER_PATH"),
    )
    return parser.parse_args()



import os
from time import sleep
from urllib.parse import quote_plus

from dotenv import load_dotenv
from selenium import webdriver
from selenium.webdriver.chrome.options import Options
from selenium.webdriver.common.by import By

load_dotenv()

CARBON_URL = (
    "https://carbon.now.sh?l={language}&code={code}"
    "&bg={background}&t={theme}&wt={wt}"
)

# in case of a slow connection it might take a bit longer to download the image
SECONDS_SLEEP_BEFORE_DOWNLOAD = int(os.environ.get("SECONDS_SLEEP_BEFORE_DOWNLOAD", 3))


def _create_carbon_url(code, **carbon_options: str) -> str:
    language = carbon_options["language"]
    background = carbon_options["background"]
    theme = carbon_options["theme"]
    wt = carbon_options["wt"]

    url = CARBON_URL.format(
        language=quote_plus(language),
        code=quote_plus(code),
        background=quote_plus(background),
        theme=quote_plus(theme),
        wt=quote_plus(wt),
    )
    return url


def create_code_image(code: str, **kwargs: str) -> None:
    """Generate a beautiful Carbon code image"""
    options = Options()
    options.headless = not bool(kwargs.get("interactive", False))

    destination = kwargs.get("destination", os.getcwd())
    prefs = {"download.default_directory": destination}
    options.add_experimental_option("prefs", prefs)

    if kwargs.get("disable-dev-shm", False):
        options.add_argument("disable-dev-shm-usage")

    url = _create_carbon_url(code, **kwargs)
    with webdriver.Chrome(kwargs["driver_path"], options=options) as driver:
        driver.get(url)
        driver.find_element(By.ID, "export-menu").click()
        driver.find_element(By.ID, "export-png").click()
        # make sure it has time to download the image
        sleep(SECONDS_SLEEP_BEFORE_DOWNLOAD)
